# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.additional_data import AdditionalData
from ..types.address_addtl_nullable import AddressAddtlNullable
from ..types.address_nullable import AddressNullable
from ..types.billing_data import BillingData
from ..types.contacts_field import ContactsField
from ..types.email import Email
from ..types.location_code import LocationCode
from ..types.mcc import Mcc
from ..types.payabli_api_response_vendors import PayabliApiResponseVendors
from ..types.payee_name import PayeeName
from ..types.remit_email import RemitEmail
from ..types.remitaddress_1 import Remitaddress1
from ..types.remitaddress_2 import Remitaddress2
from ..types.remitcity import Remitcity
from ..types.remitcountry import Remitcountry
from ..types.remitstate import Remitstate
from ..types.remitzip import Remitzip
from ..types.vendor_ein import VendorEin
from ..types.vendor_name_1 import VendorName1
from ..types.vendor_name_2 import VendorName2
from ..types.vendor_number import VendorNumber
from ..types.vendor_payment_method_string import VendorPaymentMethodString
from ..types.vendor_phone import VendorPhone
from ..types.vendor_query_record import VendorQueryRecord
from ..types.vendorstatus import Vendorstatus
from .raw_client import AsyncRawVendorClient, RawVendorClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VendorClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVendorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVendorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVendorClient
        """
        return self._raw_client

    def add_vendor(
        self,
        entry: str,
        *,
        vendor_number: typing.Optional[VendorNumber] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        address_1: typing.Optional[AddressNullable] = OMIT,
        address_2: typing.Optional[AddressAddtlNullable] = OMIT,
        billing_data: typing.Optional[BillingData] = OMIT,
        city: typing.Optional[str] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        country: typing.Optional[str] = OMIT,
        custom_field_1: typing.Optional[str] = OMIT,
        custom_field_2: typing.Optional[str] = OMIT,
        customer_vendor_account: typing.Optional[str] = OMIT,
        ein: typing.Optional[VendorEin] = OMIT,
        email: typing.Optional[Email] = OMIT,
        internal_reference_id: typing.Optional[int] = OMIT,
        location_code: typing.Optional[LocationCode] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        name_1: typing.Optional[VendorName1] = OMIT,
        name_2: typing.Optional[VendorName2] = OMIT,
        payee_name_1: typing.Optional[PayeeName] = OMIT,
        payee_name_2: typing.Optional[PayeeName] = OMIT,
        payment_method: typing.Optional[VendorPaymentMethodString] = OMIT,
        phone: typing.Optional[VendorPhone] = OMIT,
        remit_address_1: typing.Optional[Remitaddress1] = OMIT,
        remit_address_2: typing.Optional[Remitaddress2] = OMIT,
        remit_city: typing.Optional[Remitcity] = OMIT,
        remit_country: typing.Optional[Remitcountry] = OMIT,
        remit_email: typing.Optional[RemitEmail] = OMIT,
        remit_state: typing.Optional[Remitstate] = OMIT,
        remit_zip: typing.Optional[Remitzip] = OMIT,
        state: typing.Optional[str] = OMIT,
        vendor_status: typing.Optional[Vendorstatus] = OMIT,
        zip: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseVendors:
        """
        Creates a vendor in an entrypoint.

        Parameters
        ----------
        entry : str
            Entrypoint identifier.

        vendor_number : typing.Optional[VendorNumber]

        additional_data : typing.Optional[AdditionalData]

        address_1 : typing.Optional[AddressNullable]
            Vendor's address

        address_2 : typing.Optional[AddressAddtlNullable]
            Additional line for vendor's address.

        billing_data : typing.Optional[BillingData]
            Object containing vendor's bank information.

        city : typing.Optional[str]
            Vendor's city.

        contacts : typing.Optional[ContactsField]
            Array of objects describing the vendor's contacts.

        country : typing.Optional[str]
            Vendor's country.

        custom_field_1 : typing.Optional[str]
            Custom field 1 for vendor

        custom_field_2 : typing.Optional[str]
            Custom field 2 for vendor

        customer_vendor_account : typing.Optional[str]
            Account number of paypoint in the vendor side.

        ein : typing.Optional[VendorEin]

        email : typing.Optional[Email]
            Vendor's email address. Required for vCard.

        internal_reference_id : typing.Optional[int]
            Internal identifier for global vendor account.

        location_code : typing.Optional[LocationCode]

        mcc : typing.Optional[Mcc]

        name_1 : typing.Optional[VendorName1]

        name_2 : typing.Optional[VendorName2]

        payee_name_1 : typing.Optional[PayeeName]

        payee_name_2 : typing.Optional[PayeeName]

        payment_method : typing.Optional[VendorPaymentMethodString]

        phone : typing.Optional[VendorPhone]

        remit_address_1 : typing.Optional[Remitaddress1]

        remit_address_2 : typing.Optional[Remitaddress2]

        remit_city : typing.Optional[Remitcity]

        remit_country : typing.Optional[Remitcountry]

        remit_email : typing.Optional[RemitEmail]

        remit_state : typing.Optional[Remitstate]

        remit_zip : typing.Optional[Remitzip]

        state : typing.Optional[str]
            Vendor's state. Must be a 2 character state code.

        vendor_status : typing.Optional[Vendorstatus]

        zip : typing.Optional[str]
            Vendor's zip code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        from payabli import BillingData, Contacts, payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.vendor.add_vendor(
            entry="8cfec329267",
            vendor_number="1234",
            name_1="Herman's Coatings and Masonry",
            name_2="<string>",
            ein="12-3456789",
            phone="5555555555",
            email="example@email.com",
            address_1="123 Ocean Drive",
            address_2="Suite 400",
            city="Miami",
            state="FL",
            zip="33139",
            country="US",
            mcc="7777",
            location_code="MIA123",
            contacts=[
                Contacts(
                    contact_name="Herman Martinez",
                    contact_email="example@email.com",
                    contact_title="Owner",
                    contact_phone="3055550000",
                )
            ],
            billing_data=BillingData(
                id=123,
                bank_name="Country Bank",
                routing_account="123123123",
                account_number="123123123",
                type_account="Checking",
                bank_account_holder_name="Gruzya Adventure Outfitters LLC",
                bank_account_holder_type="Business",
                bank_account_function=0,
            ),
            payment_method="managed",
            vendor_status=1,
            remit_address_1="123 Walnut Street",
            remit_address_2="Suite 900",
            remit_city="Miami",
            remit_state="FL",
            remit_zip="31113",
            remit_country="US",
            payee_name_1="<string>",
            payee_name_2="<string>",
            customer_vendor_account="A-37622",
            internal_reference_id=123,
        )
        """
        _response = self._raw_client.add_vendor(
            entry,
            vendor_number=vendor_number,
            additional_data=additional_data,
            address_1=address_1,
            address_2=address_2,
            billing_data=billing_data,
            city=city,
            contacts=contacts,
            country=country,
            custom_field_1=custom_field_1,
            custom_field_2=custom_field_2,
            customer_vendor_account=customer_vendor_account,
            ein=ein,
            email=email,
            internal_reference_id=internal_reference_id,
            location_code=location_code,
            mcc=mcc,
            name_1=name_1,
            name_2=name_2,
            payee_name_1=payee_name_1,
            payee_name_2=payee_name_2,
            payment_method=payment_method,
            phone=phone,
            remit_address_1=remit_address_1,
            remit_address_2=remit_address_2,
            remit_city=remit_city,
            remit_country=remit_country,
            remit_email=remit_email,
            remit_state=remit_state,
            remit_zip=remit_zip,
            state=state,
            vendor_status=vendor_status,
            zip=zip,
            request_options=request_options,
        )
        return _response.data

    def delete_vendor(
        self, id_vendor: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseVendors:
        """
        Delete a vendor.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.vendor.delete_vendor(
            id_vendor=1,
        )
        """
        _response = self._raw_client.delete_vendor(id_vendor, request_options=request_options)
        return _response.data

    def edit_vendor(
        self,
        id_vendor: int,
        *,
        vendor_number: typing.Optional[VendorNumber] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        address_1: typing.Optional[AddressNullable] = OMIT,
        address_2: typing.Optional[AddressAddtlNullable] = OMIT,
        billing_data: typing.Optional[BillingData] = OMIT,
        city: typing.Optional[str] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        country: typing.Optional[str] = OMIT,
        custom_field_1: typing.Optional[str] = OMIT,
        custom_field_2: typing.Optional[str] = OMIT,
        customer_vendor_account: typing.Optional[str] = OMIT,
        ein: typing.Optional[VendorEin] = OMIT,
        email: typing.Optional[Email] = OMIT,
        internal_reference_id: typing.Optional[int] = OMIT,
        location_code: typing.Optional[LocationCode] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        name_1: typing.Optional[VendorName1] = OMIT,
        name_2: typing.Optional[VendorName2] = OMIT,
        payee_name_1: typing.Optional[PayeeName] = OMIT,
        payee_name_2: typing.Optional[PayeeName] = OMIT,
        payment_method: typing.Optional[VendorPaymentMethodString] = OMIT,
        phone: typing.Optional[VendorPhone] = OMIT,
        remit_address_1: typing.Optional[Remitaddress1] = OMIT,
        remit_address_2: typing.Optional[Remitaddress2] = OMIT,
        remit_city: typing.Optional[Remitcity] = OMIT,
        remit_country: typing.Optional[Remitcountry] = OMIT,
        remit_email: typing.Optional[RemitEmail] = OMIT,
        remit_state: typing.Optional[Remitstate] = OMIT,
        remit_zip: typing.Optional[Remitzip] = OMIT,
        state: typing.Optional[str] = OMIT,
        vendor_status: typing.Optional[Vendorstatus] = OMIT,
        zip: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseVendors:
        """
        Updates a vendor's information. Send only the fields you need to update.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        vendor_number : typing.Optional[VendorNumber]

        additional_data : typing.Optional[AdditionalData]

        address_1 : typing.Optional[AddressNullable]
            Vendor's address

        address_2 : typing.Optional[AddressAddtlNullable]
            Additional line for vendor's address.

        billing_data : typing.Optional[BillingData]
            Object containing vendor's bank information.

        city : typing.Optional[str]
            Vendor's city.

        contacts : typing.Optional[ContactsField]
            Array of objects describing the vendor's contacts.

        country : typing.Optional[str]
            Vendor's country.

        custom_field_1 : typing.Optional[str]
            Custom field 1 for vendor

        custom_field_2 : typing.Optional[str]
            Custom field 2 for vendor

        customer_vendor_account : typing.Optional[str]
            Account number of paypoint in the vendor side.

        ein : typing.Optional[VendorEin]

        email : typing.Optional[Email]
            Vendor's email address. Required for vCard.

        internal_reference_id : typing.Optional[int]
            Internal identifier for global vendor account.

        location_code : typing.Optional[LocationCode]

        mcc : typing.Optional[Mcc]

        name_1 : typing.Optional[VendorName1]

        name_2 : typing.Optional[VendorName2]

        payee_name_1 : typing.Optional[PayeeName]

        payee_name_2 : typing.Optional[PayeeName]

        payment_method : typing.Optional[VendorPaymentMethodString]

        phone : typing.Optional[VendorPhone]

        remit_address_1 : typing.Optional[Remitaddress1]

        remit_address_2 : typing.Optional[Remitaddress2]

        remit_city : typing.Optional[Remitcity]

        remit_country : typing.Optional[Remitcountry]

        remit_email : typing.Optional[RemitEmail]

        remit_state : typing.Optional[Remitstate]

        remit_zip : typing.Optional[Remitzip]

        state : typing.Optional[str]
            Vendor's state. Must be a 2 character state code.

        vendor_status : typing.Optional[Vendorstatus]

        zip : typing.Optional[str]
            Vendor's zip code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.vendor.edit_vendor(
            id_vendor=1,
            name_1="Theodore's Janitorial",
        )
        """
        _response = self._raw_client.edit_vendor(
            id_vendor,
            vendor_number=vendor_number,
            additional_data=additional_data,
            address_1=address_1,
            address_2=address_2,
            billing_data=billing_data,
            city=city,
            contacts=contacts,
            country=country,
            custom_field_1=custom_field_1,
            custom_field_2=custom_field_2,
            customer_vendor_account=customer_vendor_account,
            ein=ein,
            email=email,
            internal_reference_id=internal_reference_id,
            location_code=location_code,
            mcc=mcc,
            name_1=name_1,
            name_2=name_2,
            payee_name_1=payee_name_1,
            payee_name_2=payee_name_2,
            payment_method=payment_method,
            phone=phone,
            remit_address_1=remit_address_1,
            remit_address_2=remit_address_2,
            remit_city=remit_city,
            remit_country=remit_country,
            remit_email=remit_email,
            remit_state=remit_state,
            remit_zip=remit_zip,
            state=state,
            vendor_status=vendor_status,
            zip=zip,
            request_options=request_options,
        )
        return _response.data

    def get_vendor(
        self, id_vendor: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VendorQueryRecord:
        """
        Retrieves a vendor's details.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VendorQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.vendor.get_vendor(
            id_vendor=1,
        )
        """
        _response = self._raw_client.get_vendor(id_vendor, request_options=request_options)
        return _response.data


class AsyncVendorClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVendorClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVendorClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVendorClient
        """
        return self._raw_client

    async def add_vendor(
        self,
        entry: str,
        *,
        vendor_number: typing.Optional[VendorNumber] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        address_1: typing.Optional[AddressNullable] = OMIT,
        address_2: typing.Optional[AddressAddtlNullable] = OMIT,
        billing_data: typing.Optional[BillingData] = OMIT,
        city: typing.Optional[str] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        country: typing.Optional[str] = OMIT,
        custom_field_1: typing.Optional[str] = OMIT,
        custom_field_2: typing.Optional[str] = OMIT,
        customer_vendor_account: typing.Optional[str] = OMIT,
        ein: typing.Optional[VendorEin] = OMIT,
        email: typing.Optional[Email] = OMIT,
        internal_reference_id: typing.Optional[int] = OMIT,
        location_code: typing.Optional[LocationCode] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        name_1: typing.Optional[VendorName1] = OMIT,
        name_2: typing.Optional[VendorName2] = OMIT,
        payee_name_1: typing.Optional[PayeeName] = OMIT,
        payee_name_2: typing.Optional[PayeeName] = OMIT,
        payment_method: typing.Optional[VendorPaymentMethodString] = OMIT,
        phone: typing.Optional[VendorPhone] = OMIT,
        remit_address_1: typing.Optional[Remitaddress1] = OMIT,
        remit_address_2: typing.Optional[Remitaddress2] = OMIT,
        remit_city: typing.Optional[Remitcity] = OMIT,
        remit_country: typing.Optional[Remitcountry] = OMIT,
        remit_email: typing.Optional[RemitEmail] = OMIT,
        remit_state: typing.Optional[Remitstate] = OMIT,
        remit_zip: typing.Optional[Remitzip] = OMIT,
        state: typing.Optional[str] = OMIT,
        vendor_status: typing.Optional[Vendorstatus] = OMIT,
        zip: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseVendors:
        """
        Creates a vendor in an entrypoint.

        Parameters
        ----------
        entry : str
            Entrypoint identifier.

        vendor_number : typing.Optional[VendorNumber]

        additional_data : typing.Optional[AdditionalData]

        address_1 : typing.Optional[AddressNullable]
            Vendor's address

        address_2 : typing.Optional[AddressAddtlNullable]
            Additional line for vendor's address.

        billing_data : typing.Optional[BillingData]
            Object containing vendor's bank information.

        city : typing.Optional[str]
            Vendor's city.

        contacts : typing.Optional[ContactsField]
            Array of objects describing the vendor's contacts.

        country : typing.Optional[str]
            Vendor's country.

        custom_field_1 : typing.Optional[str]
            Custom field 1 for vendor

        custom_field_2 : typing.Optional[str]
            Custom field 2 for vendor

        customer_vendor_account : typing.Optional[str]
            Account number of paypoint in the vendor side.

        ein : typing.Optional[VendorEin]

        email : typing.Optional[Email]
            Vendor's email address. Required for vCard.

        internal_reference_id : typing.Optional[int]
            Internal identifier for global vendor account.

        location_code : typing.Optional[LocationCode]

        mcc : typing.Optional[Mcc]

        name_1 : typing.Optional[VendorName1]

        name_2 : typing.Optional[VendorName2]

        payee_name_1 : typing.Optional[PayeeName]

        payee_name_2 : typing.Optional[PayeeName]

        payment_method : typing.Optional[VendorPaymentMethodString]

        phone : typing.Optional[VendorPhone]

        remit_address_1 : typing.Optional[Remitaddress1]

        remit_address_2 : typing.Optional[Remitaddress2]

        remit_city : typing.Optional[Remitcity]

        remit_country : typing.Optional[Remitcountry]

        remit_email : typing.Optional[RemitEmail]

        remit_state : typing.Optional[Remitstate]

        remit_zip : typing.Optional[Remitzip]

        state : typing.Optional[str]
            Vendor's state. Must be a 2 character state code.

        vendor_status : typing.Optional[Vendorstatus]

        zip : typing.Optional[str]
            Vendor's zip code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli, BillingData, Contacts

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendor.add_vendor(
                entry="8cfec329267",
                vendor_number="1234",
                name_1="Herman's Coatings and Masonry",
                name_2="<string>",
                ein="12-3456789",
                phone="5555555555",
                email="example@email.com",
                address_1="123 Ocean Drive",
                address_2="Suite 400",
                city="Miami",
                state="FL",
                zip="33139",
                country="US",
                mcc="7777",
                location_code="MIA123",
                contacts=[
                    Contacts(
                        contact_name="Herman Martinez",
                        contact_email="example@email.com",
                        contact_title="Owner",
                        contact_phone="3055550000",
                    )
                ],
                billing_data=BillingData(
                    id=123,
                    bank_name="Country Bank",
                    routing_account="123123123",
                    account_number="123123123",
                    type_account="Checking",
                    bank_account_holder_name="Gruzya Adventure Outfitters LLC",
                    bank_account_holder_type="Business",
                    bank_account_function=0,
                ),
                payment_method="managed",
                vendor_status=1,
                remit_address_1="123 Walnut Street",
                remit_address_2="Suite 900",
                remit_city="Miami",
                remit_state="FL",
                remit_zip="31113",
                remit_country="US",
                payee_name_1="<string>",
                payee_name_2="<string>",
                customer_vendor_account="A-37622",
                internal_reference_id=123,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_vendor(
            entry,
            vendor_number=vendor_number,
            additional_data=additional_data,
            address_1=address_1,
            address_2=address_2,
            billing_data=billing_data,
            city=city,
            contacts=contacts,
            country=country,
            custom_field_1=custom_field_1,
            custom_field_2=custom_field_2,
            customer_vendor_account=customer_vendor_account,
            ein=ein,
            email=email,
            internal_reference_id=internal_reference_id,
            location_code=location_code,
            mcc=mcc,
            name_1=name_1,
            name_2=name_2,
            payee_name_1=payee_name_1,
            payee_name_2=payee_name_2,
            payment_method=payment_method,
            phone=phone,
            remit_address_1=remit_address_1,
            remit_address_2=remit_address_2,
            remit_city=remit_city,
            remit_country=remit_country,
            remit_email=remit_email,
            remit_state=remit_state,
            remit_zip=remit_zip,
            state=state,
            vendor_status=vendor_status,
            zip=zip,
            request_options=request_options,
        )
        return _response.data

    async def delete_vendor(
        self, id_vendor: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseVendors:
        """
        Delete a vendor.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendor.delete_vendor(
                id_vendor=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_vendor(id_vendor, request_options=request_options)
        return _response.data

    async def edit_vendor(
        self,
        id_vendor: int,
        *,
        vendor_number: typing.Optional[VendorNumber] = OMIT,
        additional_data: typing.Optional[AdditionalData] = OMIT,
        address_1: typing.Optional[AddressNullable] = OMIT,
        address_2: typing.Optional[AddressAddtlNullable] = OMIT,
        billing_data: typing.Optional[BillingData] = OMIT,
        city: typing.Optional[str] = OMIT,
        contacts: typing.Optional[ContactsField] = OMIT,
        country: typing.Optional[str] = OMIT,
        custom_field_1: typing.Optional[str] = OMIT,
        custom_field_2: typing.Optional[str] = OMIT,
        customer_vendor_account: typing.Optional[str] = OMIT,
        ein: typing.Optional[VendorEin] = OMIT,
        email: typing.Optional[Email] = OMIT,
        internal_reference_id: typing.Optional[int] = OMIT,
        location_code: typing.Optional[LocationCode] = OMIT,
        mcc: typing.Optional[Mcc] = OMIT,
        name_1: typing.Optional[VendorName1] = OMIT,
        name_2: typing.Optional[VendorName2] = OMIT,
        payee_name_1: typing.Optional[PayeeName] = OMIT,
        payee_name_2: typing.Optional[PayeeName] = OMIT,
        payment_method: typing.Optional[VendorPaymentMethodString] = OMIT,
        phone: typing.Optional[VendorPhone] = OMIT,
        remit_address_1: typing.Optional[Remitaddress1] = OMIT,
        remit_address_2: typing.Optional[Remitaddress2] = OMIT,
        remit_city: typing.Optional[Remitcity] = OMIT,
        remit_country: typing.Optional[Remitcountry] = OMIT,
        remit_email: typing.Optional[RemitEmail] = OMIT,
        remit_state: typing.Optional[Remitstate] = OMIT,
        remit_zip: typing.Optional[Remitzip] = OMIT,
        state: typing.Optional[str] = OMIT,
        vendor_status: typing.Optional[Vendorstatus] = OMIT,
        zip: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseVendors:
        """
        Updates a vendor's information. Send only the fields you need to update.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        vendor_number : typing.Optional[VendorNumber]

        additional_data : typing.Optional[AdditionalData]

        address_1 : typing.Optional[AddressNullable]
            Vendor's address

        address_2 : typing.Optional[AddressAddtlNullable]
            Additional line for vendor's address.

        billing_data : typing.Optional[BillingData]
            Object containing vendor's bank information.

        city : typing.Optional[str]
            Vendor's city.

        contacts : typing.Optional[ContactsField]
            Array of objects describing the vendor's contacts.

        country : typing.Optional[str]
            Vendor's country.

        custom_field_1 : typing.Optional[str]
            Custom field 1 for vendor

        custom_field_2 : typing.Optional[str]
            Custom field 2 for vendor

        customer_vendor_account : typing.Optional[str]
            Account number of paypoint in the vendor side.

        ein : typing.Optional[VendorEin]

        email : typing.Optional[Email]
            Vendor's email address. Required for vCard.

        internal_reference_id : typing.Optional[int]
            Internal identifier for global vendor account.

        location_code : typing.Optional[LocationCode]

        mcc : typing.Optional[Mcc]

        name_1 : typing.Optional[VendorName1]

        name_2 : typing.Optional[VendorName2]

        payee_name_1 : typing.Optional[PayeeName]

        payee_name_2 : typing.Optional[PayeeName]

        payment_method : typing.Optional[VendorPaymentMethodString]

        phone : typing.Optional[VendorPhone]

        remit_address_1 : typing.Optional[Remitaddress1]

        remit_address_2 : typing.Optional[Remitaddress2]

        remit_city : typing.Optional[Remitcity]

        remit_country : typing.Optional[Remitcountry]

        remit_email : typing.Optional[RemitEmail]

        remit_state : typing.Optional[Remitstate]

        remit_zip : typing.Optional[Remitzip]

        state : typing.Optional[str]
            Vendor's state. Must be a 2 character state code.

        vendor_status : typing.Optional[Vendorstatus]

        zip : typing.Optional[str]
            Vendor's zip code.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseVendors
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendor.edit_vendor(
                id_vendor=1,
                name_1="Theodore's Janitorial",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.edit_vendor(
            id_vendor,
            vendor_number=vendor_number,
            additional_data=additional_data,
            address_1=address_1,
            address_2=address_2,
            billing_data=billing_data,
            city=city,
            contacts=contacts,
            country=country,
            custom_field_1=custom_field_1,
            custom_field_2=custom_field_2,
            customer_vendor_account=customer_vendor_account,
            ein=ein,
            email=email,
            internal_reference_id=internal_reference_id,
            location_code=location_code,
            mcc=mcc,
            name_1=name_1,
            name_2=name_2,
            payee_name_1=payee_name_1,
            payee_name_2=payee_name_2,
            payment_method=payment_method,
            phone=phone,
            remit_address_1=remit_address_1,
            remit_address_2=remit_address_2,
            remit_city=remit_city,
            remit_country=remit_country,
            remit_email=remit_email,
            remit_state=remit_state,
            remit_zip=remit_zip,
            state=state,
            vendor_status=vendor_status,
            zip=zip,
            request_options=request_options,
        )
        return _response.data

    async def get_vendor(
        self, id_vendor: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VendorQueryRecord:
        """
        Retrieves a vendor's details.

        Parameters
        ----------
        id_vendor : int
            Vendor ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VendorQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendor.get_vendor(
                id_vendor=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_vendor(id_vendor, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.accounting_field import AccountingField
from ...types.additional_data_string import AdditionalDataString
from ...types.attachments import Attachments
from ...types.bill_id import BillId
from ...types.comments import Comments
from ...types.datenullable import Datenullable
from ...types.discount import Discount
from ...types.invoice_number import InvoiceNumber
from ...types.net_amountstring import NetAmountstring
from ...types.terms import Terms
from .lot_number import LotNumber


class RequestOutAuthorizeInvoiceData(UniversalBaseModel):
    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber], FieldMetadata(alias="invoiceNumber")
    ] = None
    net_amount: typing_extensions.Annotated[typing.Optional[NetAmountstring], FieldMetadata(alias="netAmount")] = None
    invoice_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="invoiceDate")] = (
        pydantic.Field(default=None)
    )
    """
    Invoice date in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    due_date: typing_extensions.Annotated[typing.Optional[Datenullable], FieldMetadata(alias="dueDate")] = (
        pydantic.Field(default=None)
    )
    """
    Invoice due date in any of the accepted formats: YYYY-MM-DD, MM/DD/YYYY.
    """

    comments: typing.Optional[Comments] = None
    lot_number: typing_extensions.Annotated[typing.Optional[LotNumber], FieldMetadata(alias="lotNumber")] = None
    bill_id: typing_extensions.Annotated[typing.Optional[BillId], FieldMetadata(alias="billId")] = None
    discount: typing.Optional[Discount] = None
    terms: typing.Optional[Terms] = None
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField1")
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField], FieldMetadata(alias="accountingField2")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString], FieldMetadata(alias="additionalData")
    ] = None
    attachments: typing.Optional[Attachments] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.file import File
from ..types.notification_query_record import NotificationQueryRecord
from ..types.payabli_api_response_notifications import PayabliApiResponseNotifications
from .raw_client import AsyncRawNotificationClient, RawNotificationClient
from .types.add_notification_request import AddNotificationRequest
from .types.update_notification_request import UpdateNotificationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NotificationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNotificationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNotificationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNotificationClient
        """
        return self._raw_client

    def add_notification(
        self, *, request: AddNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Create a new notification or autogenerated report.

        Parameters
        ----------
        request : AddNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        from payabli import (
            NotificationStandardRequest,
            NotificationStandardRequestContent,
            payabli,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notification.add_notification(
            request=NotificationStandardRequest(
                content=NotificationStandardRequestContent(
                    event_type="CreatedApplication",
                ),
                frequency="untilcancelled",
                method="web",
                owner_id="236",
                owner_type=0,
                status=1,
                target="https://webhook.site/2871b8f8-edc7-441a-b376-98d8c8e33275",
            ),
        )
        """
        _response = self._raw_client.add_notification(request=request, request_options=request_options)
        return _response.data

    def delete_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Deletes a single notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notification.delete_notification(
            n_id="1717",
        )
        """
        _response = self._raw_client.delete_notification(n_id, request_options=request_options)
        return _response.data

    def get_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationQueryRecord:
        """
        Retrieves a single notification or autogenerated report's details.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notification.get_notification(
            n_id="1717",
        )
        """
        _response = self._raw_client.get_notification(n_id, request_options=request_options)
        return _response.data

    def update_notification(
        self, n_id: str, *, request: UpdateNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Update a notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request : UpdateNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        from payabli import (
            NotificationStandardRequest,
            NotificationStandardRequestContent,
            payabli,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notification.update_notification(
            n_id="1717",
            request=NotificationStandardRequest(
                content=NotificationStandardRequestContent(
                    event_type="ApprovedPayment",
                ),
                frequency="untilcancelled",
                method="email",
                owner_id="136",
                owner_type=0,
                status=1,
                target="newemail@email.com",
            ),
        )
        """
        _response = self._raw_client.update_notification(n_id, request=request, request_options=request_options)
        return _response.data

    def get_report_file(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> File:
        """
        Gets a copy of a generated report by ID.

        Parameters
        ----------
        id : int
            Report ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        File
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.notification.get_report_file(
            id=1000000,
        )
        """
        _response = self._raw_client.get_report_file(id, request_options=request_options)
        return _response.data


class AsyncNotificationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNotificationClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNotificationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNotificationClient
        """
        return self._raw_client

    async def add_notification(
        self, *, request: AddNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Create a new notification or autogenerated report.

        Parameters
        ----------
        request : AddNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        import asyncio

        from payabli import (
            Asyncpayabli,
            NotificationStandardRequest,
            NotificationStandardRequestContent,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notification.add_notification(
                request=NotificationStandardRequest(
                    content=NotificationStandardRequestContent(
                        event_type="CreatedApplication",
                    ),
                    frequency="untilcancelled",
                    method="web",
                    owner_id="236",
                    owner_type=0,
                    status=1,
                    target="https://webhook.site/2871b8f8-edc7-441a-b376-98d8c8e33275",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_notification(request=request, request_options=request_options)
        return _response.data

    async def delete_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Deletes a single notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notification.delete_notification(
                n_id="1717",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_notification(n_id, request_options=request_options)
        return _response.data

    async def get_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> NotificationQueryRecord:
        """
        Retrieves a single notification or autogenerated report's details.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotificationQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notification.get_notification(
                n_id="1717",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_notification(n_id, request_options=request_options)
        return _response.data

    async def update_notification(
        self, n_id: str, *, request: UpdateNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseNotifications:
        """
        Update a notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request : UpdateNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseNotifications
            Success

        Examples
        --------
        import asyncio

        from payabli import (
            Asyncpayabli,
            NotificationStandardRequest,
            NotificationStandardRequestContent,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notification.update_notification(
                n_id="1717",
                request=NotificationStandardRequest(
                    content=NotificationStandardRequestContent(
                        event_type="ApprovedPayment",
                    ),
                    frequency="untilcancelled",
                    method="email",
                    owner_id="136",
                    owner_type=0,
                    status=1,
                    target="newemail@email.com",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_notification(n_id, request=request, request_options=request_options)
        return _response.data

    async def get_report_file(self, id: int, *, request_options: typing.Optional[RequestOptions] = None) -> File:
        """
        Gets a copy of a generated report by ID.

        Parameters
        ----------
        id : int
            Report ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        File
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notification.get_report_file(
                id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_report_file(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.ach_validation import AchValidation
from ..types.entrypointfield import Entrypointfield
from ..types.force_customer_creation import ForceCustomerCreation
from ..types.idempotency_key import IdempotencyKey
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_paymethod_delete import PayabliApiResponsePaymethodDelete
from ..types.payor_data_request import PayorDataRequest
from ..types.source import Source
from ..types.subdomain import Subdomain
from .types.add_method_response import AddMethodResponse
from .types.create_anonymous import CreateAnonymous
from .types.get_method_response import GetMethodResponse
from .types.request_token_storage_payment_method import RequestTokenStoragePaymentMethod
from .types.temporary import Temporary

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTokenStorageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_method(
        self,
        *,
        create_anonymous: CreateAnonymous,
        temporary: Temporary,
        ach_validation: typing.Optional[AchValidation] = None,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddMethodResponse]:
        """
        Saves a payment method for reuse. This call exchanges sensitive payment information for a token that can be used to process future transactions. The `ReferenceId` value in the response is the `storedMethodId` to use with transactions.

        Parameters
        ----------
        create_anonymous : CreateAnonymous

        temporary : Temporary

        ach_validation : typing.Optional[AchValidation]

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddMethodResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "TokenStorage/add",
            method="POST",
            params={
                "achValidation": ach_validation,
                "createAnonymous": create_anonymous,
                "forceCustomerCreation": force_customer_creation,
                "temporary": temporary,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "fallbackAuth": fallback_auth,
                "fallbackAuthAmount": fallback_auth_amount,
                "methodDescription": method_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestTokenStoragePaymentMethod, direction="write"
                ),
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddMethodResponse,
                    parse_obj_as(
                        type_=AddMethodResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_method(
        self,
        method_id: str,
        *,
        card_expiration_format: typing.Optional[int] = None,
        include_temporary: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetMethodResponse]:
        """
        Retrieves details for a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        card_expiration_format : typing.Optional[int]
            Format for card expiration dates in the response.

            Accepted values:

            - 0: default, no formatting. Expiration dates are returned in the format they're saved in.

            - 1: MMYY

            - 2: MM/YY

        include_temporary : typing.Optional[bool]
            When `true`, the request will include temporary tokens in the search and return details for a matching temporary token. The default behavior searches only for permanent tokens.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetMethodResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="GET",
            params={
                "cardExpirationFormat": card_expiration_format,
                "includeTemporary": include_temporary,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetMethodResponse,
                    parse_obj_as(
                        type_=GetMethodResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_method(
        self, method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponsePaymethodDelete]:
        """
        Deletes a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymethodDelete]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymethodDelete,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymethodDelete,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_method(
        self,
        method_id: str,
        *,
        ach_validation: typing.Optional[AchValidation] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponsePaymethodDelete]:
        """
        Updates a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        ach_validation : typing.Optional[AchValidation]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponsePaymethodDelete]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="PUT",
            params={
                "achValidation": ach_validation,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "fallbackAuth": fallback_auth,
                "fallbackAuthAmount": fallback_auth_amount,
                "methodDescription": method_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestTokenStoragePaymentMethod, direction="write"
                ),
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymethodDelete,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymethodDelete,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTokenStorageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_method(
        self,
        *,
        create_anonymous: CreateAnonymous,
        temporary: Temporary,
        ach_validation: typing.Optional[AchValidation] = None,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddMethodResponse]:
        """
        Saves a payment method for reuse. This call exchanges sensitive payment information for a token that can be used to process future transactions. The `ReferenceId` value in the response is the `storedMethodId` to use with transactions.

        Parameters
        ----------
        create_anonymous : CreateAnonymous

        temporary : Temporary

        ach_validation : typing.Optional[AchValidation]

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddMethodResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "TokenStorage/add",
            method="POST",
            params={
                "achValidation": ach_validation,
                "createAnonymous": create_anonymous,
                "forceCustomerCreation": force_customer_creation,
                "temporary": temporary,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "fallbackAuth": fallback_auth,
                "fallbackAuthAmount": fallback_auth_amount,
                "methodDescription": method_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestTokenStoragePaymentMethod, direction="write"
                ),
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddMethodResponse,
                    parse_obj_as(
                        type_=AddMethodResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_method(
        self,
        method_id: str,
        *,
        card_expiration_format: typing.Optional[int] = None,
        include_temporary: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetMethodResponse]:
        """
        Retrieves details for a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        card_expiration_format : typing.Optional[int]
            Format for card expiration dates in the response.

            Accepted values:

            - 0: default, no formatting. Expiration dates are returned in the format they're saved in.

            - 1: MMYY

            - 2: MM/YY

        include_temporary : typing.Optional[bool]
            When `true`, the request will include temporary tokens in the search and return details for a matching temporary token. The default behavior searches only for permanent tokens.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetMethodResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="GET",
            params={
                "cardExpirationFormat": card_expiration_format,
                "includeTemporary": include_temporary,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetMethodResponse,
                    parse_obj_as(
                        type_=GetMethodResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_method(
        self, method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponsePaymethodDelete]:
        """
        Deletes a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymethodDelete]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymethodDelete,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymethodDelete,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_method(
        self,
        method_id: str,
        *,
        ach_validation: typing.Optional[AchValidation] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        entry_point: typing.Optional[Entrypointfield] = OMIT,
        fallback_auth: typing.Optional[bool] = OMIT,
        fallback_auth_amount: typing.Optional[int] = OMIT,
        method_description: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[RequestTokenStoragePaymentMethod] = OMIT,
        source: typing.Optional[Source] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponsePaymethodDelete]:
        """
        Updates a saved payment method.

        Parameters
        ----------
        method_id : str
            The saved payment method ID.

        ach_validation : typing.Optional[AchValidation]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        entry_point : typing.Optional[Entrypointfield]
            Entrypoint identifier. Required for POST requests.

        fallback_auth : typing.Optional[bool]
            When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.

        fallback_auth_amount : typing.Optional[int]
            The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).

        method_description : typing.Optional[str]
            Custom description for stored payment method.

        payment_method : typing.Optional[RequestTokenStoragePaymentMethod]
            Information about the payment method for the transaction.

        source : typing.Optional[Source]
            Custom identifier to indicate the source for the request

        subdomain : typing.Optional[Subdomain]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponsePaymethodDelete]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"TokenStorage/{jsonable_encoder(method_id)}",
            method="PUT",
            params={
                "achValidation": ach_validation,
            },
            json={
                "customerData": convert_and_respect_annotation_metadata(
                    object_=customer_data, annotation=PayorDataRequest, direction="write"
                ),
                "entryPoint": entry_point,
                "fallbackAuth": fallback_auth,
                "fallbackAuthAmount": fallback_auth_amount,
                "methodDescription": method_description,
                "paymentMethod": convert_and_respect_annotation_metadata(
                    object_=payment_method, annotation=RequestTokenStoragePaymentMethod, direction="write"
                ),
                "source": source,
                "subdomain": subdomain,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponsePaymethodDelete,
                    parse_obj_as(
                        type_=PayabliApiResponsePaymethodDelete,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.file_content_ftype import FileContentFtype
from .raw_client import AsyncRawOcrClient, RawOcrClient
from .types.payabli_api_response_ocr import PayabliApiResponseOcr
from .types.type_result import TypeResult

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OcrClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOcrClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOcrClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOcrClient
        """
        return self._raw_client

    def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseOcr:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseOcr
            OCR document processing result

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.ocr.ocr_document_form(
            type_result="typeResult",
        )
        """
        _response = self._raw_client.ocr_document_form(
            type_result, ftype=ftype, filename=filename, furl=furl, f_content=f_content, request_options=request_options
        )
        return _response.data

    def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseOcr:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseOcr
            OCR document processing result

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.ocr.ocr_document_json(
            type_result="typeResult",
        )
        """
        _response = self._raw_client.ocr_document_json(
            type_result, ftype=ftype, filename=filename, furl=furl, f_content=f_content, request_options=request_options
        )
        return _response.data


class AsyncOcrClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOcrClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOcrClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOcrClient
        """
        return self._raw_client

    async def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseOcr:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseOcr
            OCR document processing result

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.ocr.ocr_document_form(
                type_result="typeResult",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.ocr_document_form(
            type_result, ftype=ftype, filename=filename, furl=furl, f_content=f_content, request_options=request_options
        )
        return _response.data

    async def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseOcr:
        """
        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseOcr
            OCR document processing result

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.ocr.ocr_document_json(
                type_result="typeResult",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.ocr_document_json(
            type_result, ftype=ftype, filename=filename, furl=furl, f_content=f_content, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .pagesize import Pagesize
from .totalpages import Totalpages
from .totalrecords import Totalrecords


class TransferSummary(UniversalBaseModel):
    """
    Examples
    --------
    from payabli import TransferSummary

    TransferSummary(
        total_pages=1,
        total_records=2,
        page_size=20,
    )
    """

    total_pages: typing_extensions.Annotated[Totalpages, FieldMetadata(alias="totalPages")]
    total_records: typing_extensions.Annotated[Totalrecords, FieldMetadata(alias="totalRecords")]
    page_size: typing_extensions.Annotated[Pagesize, FieldMetadata(alias="pageSize")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.created_at import CreatedAt
from ..types.customer_number_nullable import CustomerNumberNullable
from ..types.customer_query_records import CustomerQueryRecords
from ..types.customer_status import CustomerStatus
from ..types.email import Email
from ..types.entrypointfield import Entrypointfield
from ..types.idempotency_key import IdempotencyKey
from ..types.identifierfields import Identifierfields
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_00_responsedatanonobject import PayabliApiResponse00Responsedatanonobject
from ..types.payabli_api_response_customer_query import PayabliApiResponseCustomerQuery
from ..types.shippingaddress import Shippingaddress
from ..types.shippingaddressadditional import Shippingaddressadditional
from ..types.shippingcity import Shippingcity
from ..types.shippingcountry import Shippingcountry
from ..types.shippingstate import Shippingstate
from ..types.shippingzip import Shippingzip
from ..types.timezone import Timezone

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomerClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_customer(
        self,
        entry: Entrypointfield,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        replace_existing: typing.Optional[int] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseCustomerQuery]:
        """
        Creates a customer in an entrypoint. An identifier is required to create customer records. Change your identifier settings in Settings > Custom Fields in PartnerHub.
        If you don't include an identifier, the record is rejected.

        Parameters
        ----------
        entry : Entrypointfield

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (don't replace), 1 (replace). Default is `0`.

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseCustomerQuery]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/single/{jsonable_encoder(entry)}",
            method="POST",
            params={
                "forceCustomerCreation": force_customer_creation,
                "replaceExisting": replace_existing,
            },
            json={
                "customerNumber": customer_number,
                "customerUsername": customer_username,
                "customerPsw": customer_psw,
                "customerStatus": customer_status,
                "company": company,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "email": email,
                "address": address,
                "address1": address_1,
                "city": city,
                "state": state,
                "zip": zip,
                "country": country,
                "shippingAddress": shipping_address,
                "shippingAddress1": shipping_address_1,
                "shippingCity": shipping_city,
                "shippingState": shipping_state,
                "shippingZip": shipping_zip,
                "shippingCountry": shipping_country,
                "balance": balance,
                "timeZone": time_zone,
                "additionalFields": additional_fields,
                "identifierFields": identifier_fields,
                "createdAt": created_at,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseCustomerQuery,
                    parse_obj_as(
                        type_=PayabliApiResponseCustomerQuery,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Delete a customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CustomerQueryRecords]:
        """
        Retrieves a customer's record and details.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CustomerQueryRecords]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerQueryRecords,
                    parse_obj_as(
                        type_=CustomerQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def link_customer_transaction(
        self, customer_id: int, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Links a customer to a transaction by ID.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/link/{jsonable_encoder(customer_id)}/{jsonable_encoder(trans_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def request_consent(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Sends the consent opt-in email to the customer email address in the customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}/consent",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_customer(
        self,
        customer_id: int,
        *,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Update a customer record. Include only the fields you want to change.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="PUT",
            json={
                "customerNumber": customer_number,
                "customerUsername": customer_username,
                "customerPsw": customer_psw,
                "customerStatus": customer_status,
                "company": company,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "email": email,
                "address": address,
                "address1": address_1,
                "city": city,
                "state": state,
                "zip": zip,
                "country": country,
                "shippingAddress": shipping_address,
                "shippingAddress1": shipping_address_1,
                "shippingCity": shipping_city,
                "shippingState": shipping_state,
                "shippingZip": shipping_zip,
                "shippingCountry": shipping_country,
                "balance": balance,
                "timeZone": time_zone,
                "additionalFields": additional_fields,
                "identifierFields": identifier_fields,
                "createdAt": created_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCustomerClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_customer(
        self,
        entry: Entrypointfield,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        replace_existing: typing.Optional[int] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseCustomerQuery]:
        """
        Creates a customer in an entrypoint. An identifier is required to create customer records. Change your identifier settings in Settings > Custom Fields in PartnerHub.
        If you don't include an identifier, the record is rejected.

        Parameters
        ----------
        entry : Entrypointfield

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (don't replace), 1 (replace). Default is `0`.

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseCustomerQuery]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/single/{jsonable_encoder(entry)}",
            method="POST",
            params={
                "forceCustomerCreation": force_customer_creation,
                "replaceExisting": replace_existing,
            },
            json={
                "customerNumber": customer_number,
                "customerUsername": customer_username,
                "customerPsw": customer_psw,
                "customerStatus": customer_status,
                "company": company,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "email": email,
                "address": address,
                "address1": address_1,
                "city": city,
                "state": state,
                "zip": zip,
                "country": country,
                "shippingAddress": shipping_address,
                "shippingAddress1": shipping_address_1,
                "shippingCity": shipping_city,
                "shippingState": shipping_state,
                "shippingZip": shipping_zip,
                "shippingCountry": shipping_country,
                "balance": balance,
                "timeZone": time_zone,
                "additionalFields": additional_fields,
                "identifierFields": identifier_fields,
                "createdAt": created_at,
            },
            headers={
                "content-type": "application/json",
                "idempotencyKey": str(idempotency_key) if idempotency_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseCustomerQuery,
                    parse_obj_as(
                        type_=PayabliApiResponseCustomerQuery,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Delete a customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_customer(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CustomerQueryRecords]:
        """
        Retrieves a customer's record and details.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CustomerQueryRecords]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CustomerQueryRecords,
                    parse_obj_as(
                        type_=CustomerQueryRecords,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def link_customer_transaction(
        self, customer_id: int, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Links a customer to a transaction by ID.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/link/{jsonable_encoder(customer_id)}/{jsonable_encoder(trans_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def request_consent(
        self, customer_id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Sends the consent opt-in email to the customer email address in the customer record.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}/consent",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_customer(
        self,
        customer_id: int,
        *,
        customer_number: typing.Optional[CustomerNumberNullable] = OMIT,
        customer_username: typing.Optional[str] = OMIT,
        customer_psw: typing.Optional[str] = OMIT,
        customer_status: typing.Optional[CustomerStatus] = OMIT,
        company: typing.Optional[str] = OMIT,
        firstname: typing.Optional[str] = OMIT,
        lastname: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        email: typing.Optional[Email] = OMIT,
        address: typing.Optional[str] = OMIT,
        address_1: typing.Optional[str] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zip: typing.Optional[str] = OMIT,
        country: typing.Optional[str] = OMIT,
        shipping_address: typing.Optional[Shippingaddress] = OMIT,
        shipping_address_1: typing.Optional[Shippingaddressadditional] = OMIT,
        shipping_city: typing.Optional[Shippingcity] = OMIT,
        shipping_state: typing.Optional[Shippingstate] = OMIT,
        shipping_zip: typing.Optional[Shippingzip] = OMIT,
        shipping_country: typing.Optional[Shippingcountry] = OMIT,
        balance: typing.Optional[float] = OMIT,
        time_zone: typing.Optional[Timezone] = OMIT,
        additional_fields: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        identifier_fields: typing.Optional[Identifierfields] = OMIT,
        created_at: typing.Optional[CreatedAt] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]:
        """
        Update a customer record. Include only the fields you want to change.

        Parameters
        ----------
        customer_id : int
            Payabli-generated customer ID. Maps to "Customer ID" column in PartnerHub.

        customer_number : typing.Optional[CustomerNumberNullable]

        customer_username : typing.Optional[str]
            Customer username for customer portal

        customer_psw : typing.Optional[str]
            Customer password for customer portal

        customer_status : typing.Optional[CustomerStatus]

        company : typing.Optional[str]
            Company name

        firstname : typing.Optional[str]
            Customer first name

        lastname : typing.Optional[str]
            Customer last name

        phone : typing.Optional[str]
            Customer phone number

        email : typing.Optional[Email]
            Customer email address.

        address : typing.Optional[str]
            Customer address

        address_1 : typing.Optional[str]
            Additional customer address

        city : typing.Optional[str]
            Customer city

        state : typing.Optional[str]
            Customer State

        zip : typing.Optional[str]
            Customer postal code

        country : typing.Optional[str]
            Customer country in ISO-3166-1 alpha 2 format. See https://en.wikipedia.org/wiki/ISO_3166-1 for reference.

        shipping_address : typing.Optional[Shippingaddress]

        shipping_address_1 : typing.Optional[Shippingaddressadditional]

        shipping_city : typing.Optional[Shippingcity]

        shipping_state : typing.Optional[Shippingstate]

        shipping_zip : typing.Optional[Shippingzip]

        shipping_country : typing.Optional[Shippingcountry]

        balance : typing.Optional[float]
            Customer balance.

        time_zone : typing.Optional[Timezone]

        additional_fields : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Additional Custom fields in format "key":"value".

        identifier_fields : typing.Optional[Identifierfields]

        created_at : typing.Optional[CreatedAt]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponse00Responsedatanonobject]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Customer/{jsonable_encoder(customer_id)}",
            method="PUT",
            json={
                "customerNumber": customer_number,
                "customerUsername": customer_username,
                "customerPsw": customer_psw,
                "customerStatus": customer_status,
                "company": company,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "email": email,
                "address": address,
                "address1": address_1,
                "city": city,
                "state": state,
                "zip": zip,
                "country": country,
                "shippingAddress": shipping_address,
                "shippingAddress1": shipping_address_1,
                "shippingCity": shipping_city,
                "shippingState": shipping_state,
                "shippingZip": shipping_zip,
                "shippingCountry": shipping_country,
                "balance": balance,
                "timeZone": time_zone,
                "additionalFields": additional_fields,
                "identifierFields": identifier_fields,
                "createdAt": created_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponse00Responsedatanonobject,
                    parse_obj_as(
                        type_=PayabliApiResponse00Responsedatanonobject,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enabled import Enabled
from .method_element_settings import MethodElementSettings
from .methods_list import MethodsList
from .order import Order


class MethodElement(UniversalBaseModel):
    all_methods_checked: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="allMethodsChecked")
    ] = pydantic.Field(default=None)
    """
    Flag indicating if all allowed payment methods will be pre-selected.
    """

    enabled: typing.Optional[Enabled] = None
    header: typing.Optional[str] = pydantic.Field(default=None)
    """
    Header text for section
    """

    methods: typing.Optional[MethodsList] = None
    order: typing.Optional[Order] = None
    settings: typing.Optional[MethodElementSettings] = pydantic.Field(default=None)
    """
    Settings for wallet payment methods.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.customeridtrans import Customeridtrans
from ...types.referenceidtrans import Referenceidtrans
from ...types.result_code import ResultCode
from ...types.resulttext import Resulttext


class CaptureAllOutResponseResponseDataItem(UniversalBaseModel):
    customer_id: typing_extensions.Annotated[typing.Optional[Customeridtrans], FieldMetadata(alias="CustomerId")] = (
        pydantic.Field(default=None)
    )
    """
    Internal unique Id of vendor owner of transaction. Returns `0` if the transaction wasn't assigned to an existing vendor or no vendor was created.
    """

    reference_id: typing_extensions.Annotated[typing.Optional[Referenceidtrans], FieldMetadata(alias="ReferenceId")] = (
        None
    )
    result_code: typing_extensions.Annotated[typing.Optional[ResultCode], FieldMetadata(alias="ResultCode")] = None
    result_text: typing_extensions.Annotated[typing.Optional[Resulttext], FieldMetadata(alias="ResultText")] = (
        pydantic.Field(default=None)
    )
    """
    Text describing the result. 
    If `ResultCode`` = 1, returns 'Authorized'. 
    If `ResultCode` = 2 or 3, this contains the cause of the decline.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
